#!/usr/bin/env python3
"""
Summary.

    Prints iam usernames from local awscli configuration.
    A usernames may be omitted from the output simply by
    listing them with a space between them after the call:

    $ python3 iam_users.py default

    Will print all iam usernames in the local configuration
    except the default user (username "default")

"""
import os
import sys
import inspect

try:
    from configparser import ConfigParser
except Exception:
    print('unable to import configParser library. Exit')
    sys.exit(1)


# --- declarations  --------------------------------------------------------------------------------


def print_array(content, args):
    for x in content.sections():
        if x in args:
            continue
        else:
            print(x + ' ', end='')


def shared_credentials_location():
    """
    Summary:
        Discover alterate location for awscli shared credentials file
    Returns:
        TYPE: str, Full path of shared credentials file, if exists
    """
    if 'AWS_SHARED_CREDENTIALS_FILE' in os.environ:
        return os.environ['AWS_SHARED_CREDENTIALS_FILE']
    return ''


def awscli_profiles():
    """Returns IAM usernames from local awscli configuration"""
    if os.path.isfile(config_file):
        config.read(config_file)
    else:
        print('[WARN]: awscli configuration file not found on local filesystem.')
        sys.exit(1)

    for profile in config.sections():
        if 'role_arn' in config[profile].keys():
            config.pop(profile)
    return config


def print_profiles(config, args):
    """Execution when no parameters provided"""
    try:
        print_array(config, args)
    except OSError as e:
        print('{}: OSError: {}'.format(inspect.stack(0)[3], e))
        return False
    return True


# --- main --------------------------------------------------------------------------------


# globals
home = os.environ.get('HOME')
config_file = shared_credentials_location() or home + '/.aws/credentials'
config = ConfigParser()

modified_config = awscli_profiles()
sys.exit(print_profiles(modified_config, sys.argv[1:]))



"""
Display of help menu contents and available options

"""
from rulemanager import Colors, PACKAGE

c = Colors()


def help_menu():
    """
    Summary.

        Command line parameter options (Help Menu)

    """
    # formatting
    act = c.ORANGE                         # accent highlight (bright orange)
    bcy = c.BRIGHT_CYAN
    bd = c.BOLD + c.BRIGHT_WHITE           # title formatting
    rst = c.RESET
    lbct = bcy + '[' + rst
    rbct = bcy + ']' + rst
    ctr = bcy + '|' + rst

    menu = '''
                      ''' + bd + PACKAGE + rst + ''' help contents

  ''' + bd + '''DESCRIPTION''' + rst + '''

        Rulemanager is a command line utility used to display, sort,
        and update CloudWatch rules in an AWS Account.

  ''' + bd + '''SYNOPSIS''' + rst + '''

        $ ''' + act + PACKAGE + rst + ' --view ' + lbct + ' --keyword <WORD> ' + rbct + ' ' + ctr + ' --set-time <UTC> '  + '''

                         -v, --view
                        [-d, --disable ]
                        [-e, --enable  ]
                        [-k, --keyword <value>  ]
                        [-p, --profile <value>  ]
                        [-r, --region <value>   ]
                        [-s, --set-time <value> ]
                        [-d, --debug  ]
                        [-h, --help   ]

  ''' + bd + '''OPTIONS''' + rst + '''

        ''' + bd + '''-D''' + rst + ''', ''' + bd + '''--debug''' + rst + ''':  Debugging mode,  verbose output for bug tracing.

        ''' + bd + '''-d''' + rst + ''', ''' + bd + '''--disable''' + rst + ''':  Disable a specific CloudWatch  rule or set of
            rules. Must use with the --keyword parameter.

        ''' + bd + '''-e''' + rst + ''', ''' + bd + '''--enable''' + rst + ''':  Enable a specific rule or rules.  Must use with
            the --keyword parameter.

        ''' + bd + '''-h''' + rst + ''', ''' + bd + '''--help''' + rst + ''':  Print this help menu and associated option info.

        ''' + bd + '''-k''' + rst + ''', ''' + bd + '''--keyword''' + rst + ''': CloudWatch rules may be filtered by specifying
            a search keyword that may be found in the rule name.

        ''' + bd + '''-p''' + rst + ''', ''' + bd + '''--profile''' + rst + ''':  AWS identity and access profile username with
            appropriate IAM permissions to configure CloudWatch rules.

        ''' + bd + '''-r''' + rst + ''', ''' + bd + '''--region''' + rst + ''':  AWS region where a specific CloudWatch rule of
            interest may be found.

        ''' + bd + '''-s''' + rst + ''', ''' + bd + '''--set-time''' + rst + ''': Reset the rule trigger time to new value.  All
            trigger times in the UTC (GMT) timezone.

        ''' + bd + '''-v''' + rst + ''', ''' + bd + '''--view''' + rst + ''':  Display all CloudWatch rules in the specified AWS
            AWS region.

        ''' + bd + '''-V''' + rst + ''', ''' + bd + '''--version''' + rst + ''':  Print app package version  and copyright info.
    '''
    print(menu)
    return True



"""
Table Generation Module:
    - Constructs table of data
    - Displays table with appropriate cli positioning

Module Functions:
    - setup_table:
        Main module function controller for constructing a table of data
    - display_table:
        renders VeryPrettyTable table to cli stdout
    - print_header:
        Separate module function for contructing the table title row (first row)
    - _postprocessing:
        Facility for initating any post-table construction operations
"""

import os
import sys
import inspect
import unicodedata

# 3rd party
from veryprettytable import VeryPrettyTable
from cron_descriptor import get_description, ExpressionDescriptor
from libtools.js import export_iterobject
from libtools import stdout_message, logd, Colors


c = Colors()


try:
    from libtools.oscodes_unix import exit_codes
    os_type = 'Linux'
    splitchar = '/'                             # character for splitting paths (linux)
    text = c.BRIGHT_CYAN
except Exception:
    from libtools.oscodes_win import exit_codes    # non-specific os-safe codes
    os_type = 'Windows'
    splitchar = '\\'                            # character for splitting paths (windows)
    text = c.CYAN


# universal colors; unicode symbols
yl = c.YELLOW + c.BOLD
fs = c.GOLD3
bd = c.BOLD
titlec = c.BRIGHT_WHITE
gn = c.BRIGHT_GREEN
btext = text + c.BOLD
bdwt = c.BOLD + c.BRIGHT_WHITE
dgray = c.DARK_GRAY1
frame = text
ub = c.UNBOLD
bullet = unicodedata.lookup('bullet')
rst = c.RESET


tablespec = {
    'border': True,
    'header': True,
    'padding': 2,
    'field_max_width': 70
}

column_widths = {
    'name': 31,
    'expression': 36,
    'state': 8,
    'arn': 70
}


def display_table(table, tabspaces=4):
    """
    Print Table Object offset from left by tabspaces
    """
    indent = ('\t').expandtabs(tabspaces)
    table_str = table.get_string()
    for e in table_str.split('\n'):
        print(indent + frame + e)
    sys.stdout.write(Colors.RESET + '\n\n')
    return True


def print_header(title, indent=4, spacing=4):
    """
    Paints title header grid of a vpt Table
    """
    divbar = frame + '-'
    upbar = frame + '|' + rst
    ltab = '\t'.expandtabs(indent)              # lhs indentation of title bar
    spac = '\t'.expandtabs(7)                   # rhs indentation of legend from divider bar
    tab3 = '\t'.expandtabs(3)                   # space between legend items
    tab4 = '\t'.expandtabs(4)                   # space between legend items
    tab5 = '\t'.expandtabs(5)                   # space between legend items
    tab6 = '\t'.expandtabs(6)                   # space between legend items
    # output header
    print('\n\n')
    print(tab4, end='')
    print(divbar * 91, end='\n')
    print(tab4 + '|' + ' ' * 89 + '|', end='\n')
    print('{}{}'.format(tab4 + '|' + tab3 * 5, title))
    print(tab4 + upbar + rst + ' ' + tab3 * 29 + frame + ' |' + rst)
    return True


def spacing(days):
    s = ''
    for digit in str(days):
        s = s + ' '
    return s


def _postprocessing():
    return True


def setup_table(rule_list, aws_account, rgn):
    """
    Renders Table containing data elements via cli stdout
    """
    # guarding clause in case no cw rules in region
    if len(rule_list) < 1:
        msg = 'No CloudWatch rules found in region {} of AWS Account {}.'.format(rgn, aws_account)
        stdout_message(msg)
        return _postprocessing()

    # setup table
    x = VeryPrettyTable(
            border=tablespec['border'],
            header=tablespec['header'],
            padding_width=tablespec['padding']
        )

    title_cell1 = '           Rule Name'              # forward leading spaces req to center title
    title_cell2 = '       Trigger Frequency (UTC)'
    title_cell3 = 'State'

    x.field_names = [
        titlec + title_cell1 + frame,
        titlec + title_cell2 + frame,
        titlec + title_cell3 + frame,
    ]

    # cell max width
    x.max_width[titlec + title_cell1 + frame] = column_widths['name']
    x.max_width[titlec + title_cell2 + frame] = column_widths['expression']
    x.max_width[titlec + title_cell3 + frame] = column_widths['state']

    # cell min = max width
    x.min_width[titlec + title_cell1 + frame] = column_widths['name']
    x.min_width[titlec + title_cell2 + frame] = column_widths['expression']
    x.min_width[titlec + title_cell3 + frame] = column_widths['state']

    # cell alignment
    x.align[titlec + title_cell1 + frame] = 'l'
    x.align[titlec + title_cell2 + frame] = 'l'
    x.align[titlec + title_cell3  + frame] = 'c'

    for rule in rule_list:

        try:

            # convert cron expression to human-readable
            root = str(ExpressionDescriptor(rule['ScheduleExpression'][5:-2]))

            # prepend to expression root if daily event
            expression = 'Daily ' + root if (len(root) == 11) and ('At' in root) else root

        except Exception:
            expression = rule['ScheduleExpression'] if rule.get('ScheduleExpression') else 'N/A'

        # populate table
        rule_name = rule['Name'] if len(rule['Name']) < 29 else rule['Name'][:28]
        rule_expression = c.BOLD + c.BRIGHT_WHITE + expression + rst
        rule_state = (c.BRIGHT_GREEN if rule['State'] == 'ENABLED' else c.RED) + rule['State'] + rst

        x.add_row(
            [
                rst + ' ' + c.ORANGE +  bullet + ' ' + c.BRIGHT_BLUE + rule_name + rst + frame,
                rst + c.ORANGE + bullet + ' ' + rule_expression + frame,
                rst + rule_state + frame,
            ]
        )

    # Table
    account = aws_account
    vtab_int = 10
    vtab = '\t'.expandtabs(vtab_int)
    init = rst + bd + frame
    acct = rst + account + frame
    rgn_c = rst + rgn + frame
    bar =  ' ' + vtab
    msg = '{}CloudWatch Rules | {}AWS ACcount: {}, Region: {}{}|{}'.format(init, ub, acct, rgn_c, bar, rst)
    print_header(title=msg, indent=10, spacing=vtab_int)
    display_table(x, tabspaces=4)
    return _postprocessing()



#!/usr/bin/env python3
"""
Summary.

    Prints AWS region codes by quering the Amazon APIs

        $ python3 regions.py default

Returns:
    AWS region codes (str). Example:  'us-east-1'
"""
import os
import sys
import inspect
import datetime
from botocore.exceptions import ClientError
import boto3
from rulemanager.config import _region_list

try:
    from configparser import ConfigParser
except Exception:
    print('unable to import configParser library. Exit')
    sys.exit(1)


PACKAGE = 'rulemanager'
CONFIG_DIR = os.getenv('HOME') + '/.config/' + PACKAGE
REFERENCE = CONFIG_DIR + '/' + 'regions.list'
MAX_AGE_DAYS = 3


# --- declarations  --------------------------------------------------------------------------------


def authenticated(profile):
    """
        Tests generic authentication status to AWS Account

    Args:
        :profile (str): iam user name from local awscli configuration

    Returns:
        TYPE: bool, True (Authenticated)| False (Unauthenticated)

    """
    try:

        session = boto3.Session(profile_name=profile)
        sts_client = session.client('sts')
        httpstatus = sts_client.get_caller_identity()['ResponseMetadata']['HTTPStatusCode']
        if httpstatus == 200:
            return True

    except ClientError as e:
        return False
    except Exception as e:
        fx = inspect.stack()[0][3]
        logger.exception('{}: Unknown error attempting to authenicate to AWS: {}'.format(fx, e))
        return False
    return False


def file_age(filepath, unit='seconds'):
    """
    Summary.

        Calculates file age in seconds

    Args:
        :filepath (str): path to file
        :unit (str): unit of time measurement returned.
    Returns:
        age (int)
    """
    ctime = os.path.getctime(filepath)
    dt = datetime.datetime.fromtimestamp(ctime)
    now = datetime.datetime.utcnow()
    delta = now - dt
    if unit == 'days':
        return round(delta.days, 2)
    elif unit == 'hours':
        round(delta.seconds / 3600, 2)
    return round(delta.seconds, 2)


def get_regions(profile=None):
    """ Return list of all regions """
    try:
        if profile is None:
            profile = 'default'

        if authenticated(profile):
            session = boto3.Session(profile_name=profile)
            client = session.client('ec2')
        else:
            return None

    except ClientError as e:
        logger.exception(
            '%s: Boto error while retrieving regions (%s)' %
            (inspect.stack()[0][3], str(e)))
        raise e
    return [x['RegionName'] for x in client.describe_regions()['Regions']]


def print_array(args):
    for x in args:
        print(x.strip() + ' ', end='')


def shared_credentials_location():
    """
    Summary:
        Discover alterate location for awscli shared credentials file
    Returns:
        TYPE: str, Full path of shared credentials file, if exists
    """
    if 'AWS_SHARED_CREDENTIALS_FILE' in os.environ:
        return os.environ['AWS_SHARED_CREDENTIALS_FILE']
    return ''


def print_profiles(config, args):
    """Execution when no parameters provided"""
    try:
        print_array(config, args)
    except OSError as e:
        print('{}: OSError: {}'.format(inspect.stack(0)[3], e))
        return False
    return True


def read(fname):
    basedir = os.path.dirname(sys.argv[0])
    return open(os.path.join(basedir, fname)).read()


def write_file(object, filepath):
    try:
        with open(filepath, 'w') as f1:
            for i in object:
                f1.write(i + '\n')
    except OSError as e:
        print('Error: {}'.format(e))


# --- main --------------------------------------------------------------------------------


PROFILE = None


# globals
if len(sys.argv) > 1:
    if sys.argv[1] == '--profile':
        PROFILE = sys.argv[2]
if PROFILE is None:
    PROFILE = 'default'

if os.path.exists(REFERENCE) and file_age(REFERENCE, 'days') < MAX_AGE_DAYS:
    r = read(REFERENCE)
    regions = [x for x in r.split('\n') if x]
    sys.exit(print_array(regions))
else:
    regions = get_regions(profile=PROFILE) or _region_list()
    print_array(regions)
    write_file(regions, REFERENCE)
    sys.exit(0)



"""

Amazon CloudWatch Rule Converter, GPL v3 License

Copyright (c) 2020-2021 Blake Huber

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""
import os
import sys
import re
import datetime
import json
import inspect
import argparse
import boto3
import threading
from botocore.exceptions import ClientError
from libtools import stdout_message, logd
from libtools.js import export_iterobject
from libtools.oscodes_unix import exit_codes
from rulemanager.table import setup_table
from rulemanager.config import script_config
from rulemanager.about import about_object
from rulemanager.help import help_menu
from rulemanager import __version__, PACKAGE, logger

# globals
module = os.path.basename(__file__)


def account_number(profile):
    """Retrieves AWS Account Number"""
    session = boto3.Session(profile_name=profile)
    client = session.client('sts')
    return client.get_caller_identity()['Account']


def get_target_rules(profile, rgn, keyword='all'):
    """
    Returns CloudWatch Rules if exist based on search keyword
    """
    session = boto3.Session(profile_name=profile)
    client = session.client('events', region_name=rgn)
    r = client.list_rules()['Rules']
    if keyword == 'all':
        return [x for x in r]
    return [x for x in r if re.search(str(keyword), x['Name'], re.IGNORECASE)]


def _debug_output(*args):
    """additional verbose information output"""
    for arg in args:
        if os.path.isfile(arg):
            print('Filename {}'.format(arg.strip(), 'lower'))
        elif str(arg):
            print('String {} = {}'.format(getattr(arg.strip(), 'title'), arg))


def default_region():
    """Returns default AWS Region, if set"""
    return os.getenv('AWS_DEFAULT_REGION') or 'us-east-1'


def _get_regions():
    client = boto3.client('ec2')
    return [x['RegionName'] for x in client.describe_regions()['Regions']]


def format_pricefile(key):
    """Adds path delimiter and color formatting to output artifacts"""
    region = key.split('/')[0]
    pricefile = key.split('/')[1]
    delimiter = '/'
    return region + delimiter + pricefile


def package_version():
    """
    Prints package version and requisite PACKAGE info
    """
    print(about_object)
    return True


def source_environment(env_variable):
    """
    Sources all environment variables
    """
    return {
        'duration_days': read_env_variable('DEFAULT_DURATION'),
        'page_size': read_env_variable('PAGE_SIZE', 700),
        'bucket': read_env_variable('S3_BUCKET', None)
    }.get(env_variable, None)


def options(parser, help_menu=False):
    """
    Summary:
        parse cli parameter options

    Returns:
        TYPE: argparse object, parser argument set

    """
    parser.add_argument("-D", "--debug", dest='debug', action='store_true', default=False, required=False)
    parser.add_argument("-d", "--disable", dest='disable', action='store_true', default=False, required=False)
    parser.add_argument("-e", "--enable", dest='enable', action='store_true', default=False, required=False)
    parser.add_argument("-h", "--help", dest='help', action='store_true', default=False, required=False)
    parser.add_argument("-k", "--keyword", dest='keyword', default=None, nargs='?', type=str, required=False)
    parser.add_argument("-p", "--profile", dest='profile', default='default', nargs='?', type=str, required=False)
    parser.add_argument("-r", "--region", dest='region', default=default_region(), nargs='?', type=str, required=False)
    parser.add_argument("-v", "--view", dest='view', action='store_true', default=False, required=False)
    parser.add_argument("-V", "--version", dest='version', action='store_true', default=False, required=False)
    return parser.parse_known_args()


def enable_rule(profile, rgn, keyword):
    """Enable (activate) specified CloudWatch rules"""
    session = boto3.Session(profile_name=profile)
    client = session.client('events', region_name=rgn)

    # extract cw rule names
    names = [x['Name'] for x in get_target_rules(profile, rgn, keyword)]

    if len(names) == 0:
        stdout_message('No CloudWatch rules found to enable', prefix='WARN')
        return False

    for name in names:
        try:
            r = client.enable_rule(Name=name)
            httpstatus = r['ResponseMetadata']['HTTPStatusCode']
            if str(httpstatus).startswith('20'):
                stdout_message('CloudWatch rule {} set to enabled.'.format(name))
            else:
                stdout_message('Failed to set CloudWatch rule {} to enabled.'.format(name), prefix='WARN')
        except ClientError as e:
            logger.exception('Boto ClientError: {}'.format(e))
            continue
    return True


def disable_rule(profile, rgn, keyword):
    """Enable (activate) specified CloudWatch rules"""
    session = boto3.Session(profile_name=profile)
    client = session.client('events', region_name=rgn)

    # extract cw rule names
    names = [x['Name'] for x in get_target_rules(profile, rgn, keyword)]

    if len(names) == 0:
        stdout_message('No CloudWatch rules found to enable', prefix='WARN')
        return False

    for name in names:
        try:
            r = client.disable_rule(Name=name)
            httpstatus = r['ResponseMetadata']['HTTPStatusCode']
            if str(httpstatus).startswith('20'):
                stdout_message('CloudWatch rule {} disabled.'.format(name))
            else:
                stdout_message('Failed to disable CloudWatch rule {}.'.format(name), prefix='WARN')
        except ClientError as e:
            logger.exception('Boto ClientError: {}'.format(e))
            continue
    return True


def init():
    """
    Initialize spot price operations; process command line parameters
    """

    parser = argparse.ArgumentParser(add_help=False)

    try:

        args, unknown = options(parser)

    except Exception as e:
        stdout_message(str(e), 'ERROR')
        return False

    if args.debug:
        stdout_message('PACKAGE: {}'.format(PACKAGE), prefix='DBUG')
        stdout_message('module: {}'.format(module), prefix='DBUG')
        stdout_message('module_path: {}'.format(os.path.join(PACKAGE, module)), prefix='DBUG')
        # log status
        logger.info('Environment variable status:')
        logger.info('\t- REGION: {}'.format(args.region))
        logger.info('\t- DBUGMODE is: {}'.format(args.debug))

    if args.help or (len(sys.argv) == 1):
        return help_menu()

    elif args.version:
        return package_version()

    elif args.view and args.keyword:
        account = account_number(args.profile)
        return setup_table(
                    get_target_rules(args.profile, args.region, args.keyword),
                    account,
                    args.region
                )

    elif args.view:
        account = account_number(args.profile)
        return setup_table(get_target_rules(args.profile, args.region), account, args.region)

    elif (args.enable and args.keyword) and not args.disable:
        account = account_number(args.profile)
        setup_table(
                    get_target_rules(args.profile, args.region, args.keyword),
                    account,
                    args.region
                )
        if enable_rule(args.profile, args.region, args.keyword):
            return setup_table(
                        get_target_rules(args.profile, args.region, args.keyword),
                        account,
                        args.region
                    )
        return False

    elif (args.disable and args.keyword) and not args.enable:
        account = account_number(args.profile)
        setup_table(
                    get_target_rules(args.profile, args.region, args.keyword),
                    account,
                    args.region
                )
        if disable_rule(args.profile, args.region, args.keyword):
            return setup_table(
                        get_target_rules(args.profile, args.region, args.keyword),
                        account,
                        args.region
                    )
        return False



"""
Build Script local logging and configuration variables

    - Python3 only
    - Requirement to set artifact which contains PACKAGE name
    - log_mode sets type of logging (i.e. 'STREAM' or 'FILE')

"""
import os
import sys
import subprocess


artifact = 'DESCRIPTION.rst'
enable_logging = True
log_filename = ''
log_path = ''
log_mode = 'STREAM'


def _region_list():
    """Returns a static list of AWS regions for bash completion"""
    return [
        'eu-north-1',
        'ap-south-1',
        'eu-west-3',
        'eu-west-2',
        'eu-west-1',
        'ap-northeast-2',
        'ap-northeast-1',
        'sa-east-1',
        'ca-central-1',
        'ap-southeast-1',
        'ap-southeast-2',
        'eu-central-1',
        'us-east-1',
        'us-east-2',
        'us-west-1',
        'us-west-2'
    ]


script_config = {
    "PROJECT": {
        "PACKAGE": 'rulemanager',
    },
    "LOGGING": {
        "ENABLE_LOGGING": enable_logging,
        "LOG_FILENAME": log_filename,
        "LOG_PATH": log_path,
        "LOG_MODE": log_mode,
        "SYSLOG_FILE": False
    }
}



from rulemanager._version import __version__ as version
from rulemanager.config import script_config


__author__ = 'Blake Huber'
__version__ = version
__email__ = "blakeca00@gmail.com"


PACKAGE = script_config['PROJECT']['PACKAGE']

try:


    from libtools import Colors
    from libtools import logd

    # global logger
    logd.local_config = script_config
    logger = logd.getLogger(__version__)


except Exception:
    pass



"""

rulemanager :  Copyright 2018-2020, Blake Huber

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

see: https://www.gnu.org/licenses/#GPL

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
contained in the program LICENSE file.

"""

import os
import sys
import platform
import subprocess
import inspect
from shutil import which
from shutil import copy2 as copyfile
from setuptools import setup, find_packages
from setuptools.command.develop import develop
from setuptools.command.install import install
import getpass
from codecs import open
import rulemanager


requires = [
    'boto3>=1.12.1',
    'libtools>=0.3.3',
    'cron-descriptor'
]


_project = 'rulemanager'
_root = os.path.abspath(os.path.dirname(__file__))
_iam_script = 'iam_identities.py'
_rgn_script = 'regions.py'
_comp_fname = 'rulemanager-completion.bash'


def _git_root():
    """Returns root directory of git repository"""
    cmd = 'git rev-parse --show-toplevel 2>/dev/null'
    return subprocess.getoutput(cmd).strip()


def _root_user():
    """
    Checks localhost root or sudo access.  Returns bool
    """
    if os.geteuid() == 0:
        return True
    elif subprocess.getoutput('echo $EUID') == '0':
        return True
    return False


def _user():
    """Returns username of caller"""
    return getpass.getuser()


def _set_pythonpath():
    """
    Temporarily reset PYTHONPATH to prevent home dir = python module home
    """
    os.environ['PYTHONPATH'] = '/'


def create_artifact(object_path, type):
    """Creates post install filesystem artifacts"""
    if type == 'file':
        with open(object_path, 'w') as f1:
            f1.write(sourcefile_content())
    elif type == 'dir':
        os.makedirs(object_path)


def _install_root():
    """Filsystem installed location of program modules"""
    if not _root_user():
        return os.path.abspath(os.path.dirname(rulemanager.__file__))
    return os.path.join(inspect.getsourcefile(setup).split('setuptools')[0], _project)


def module_src():
    """Alt method of finding filsystem location of Python3 modules"""
    bin_path = which('python3.6') or which('python3.7')
    bin = os.path.split(bin_path)[1]
    if 'local' in bin_path:
        return '/usr/local/lib/' + bin + '/site-packages'
    return '/usr/lib/' + bin + '/site-packages'


def os_parityPath(path):
    """
    Converts unix paths to correct windows equivalents.
    Unix native paths remain unchanged (no effect)
    """
    path = os.path.normpath(os.path.expanduser(path))
    if path.startswith('\\'):
        return 'C:' + path
    return path


class PostInstallDevelop(develop):
    """ post-install, development """
    def run(self):
        subprocess.check_call("bash scripts/post-install-dev.sh".split())
        develop.run(self)


class PostInstall(install):
    """
    Summary.

        Postinstall script to place bash completion artifacts
        on local filesystem

    """
    def valid_os_shell(self):
        """
        Summary.

            Validates install environment for Linux and Bash shell

        Returns:
            Success | Failure, TYPE bool

        """
        if platform.system() == 'Windows':
            return False
        elif which('bash'):
            return True
        elif 'bash' in subprocess.getoutput('echo $SHELL'):
            return True
        return False

    def run(self):
        """
        Summary.

            Executes post installation configuration only if correct
            environment detected

        """
        if self.valid_os_shell():

            completion_file = os.path.join(user_home(), '.bash_completion')
            completion_dir = os.path.join(user_home(), '.bash_completion.d')
            config_dir = os.path.join(user_home(), '.config', _project)

            if not os.path.exists(os_parityPath(completion_file)):
                create_artifact(os_parityPath(completion_file), 'file')
            if not os.path.exists(os_parityPath(completion_dir)):
                create_artifact(os_parityPath(completion_dir), 'dir')
            if not os.path.exists(os_parityPath(config_dir)):
                create_artifact(os_parityPath(config_dir), 'dir')

            ## ensure installation of home directory artifacts (data_files) ##

            # bash_completion; (overwrite if exists)
            copyfile(
                os_parityPath(os.path.join('bash', _comp_fname)),
                os_parityPath(os.path.join(completion_dir, _comp_fname))
            )
            # configuration files: excluded file types
            if not os.path.exists(os_parityPath(os.path.join(config_dir, _iam_script))):
                copyfile(
                    os_parityPath(os.path.join('bash', _iam_script)),
                    os_parityPath(os.path.join(config_dir, _iam_script))
                )
            # configuration files: excluded directories
            if not os.path.exists(os_parityPath(os.path.join(config_dir, _rgn_script))):
                copyfile(
                    os_parityPath(os.path.join('bash', _rgn_script)),
                    os_parityPath(os.path.join(config_dir, _rgn_script))
                )
        install.run(self)


class PostInstallRoot(install):
    """
    Summary.

        Postinstall script to place bash completion artifacts
        on local filesystem

    """
    def valid_os_shell(self):
        """
        Summary.

            Validates install environment for Linux and Bash shell

        Returns:
            Success | Failure, TYPE bool

        """
        if platform.system() == 'Windows':
            return False
        elif which('bash'):
            return True
        elif 'bash' in subprocess.getoutput('echo $SHELL'):
            return True
        return False

    def run(self):
        """
        Summary.

            Executes post installation configuration only if correct
            environment detected

        """
        # bash shell + root user
        if self.valid_os_shell():

            completion_dir = '/etc/bash_completion.d'
            config_dir = os.path.join(_install_root(), 'config')

            if not os.path.exists(os_parityPath(config_dir)):
                create_artifact(os_parityPath(config_dir), 'dir')

            ## ensure installation of home directory artifacts (data_files) ##

            # bash_completion; (overwrite if exists)
            copyfile(
                os_parityPath(os.path.join('bash', _comp_fname)),
                os_parityPath(os.path.join(completion_dir, _comp_fname))
            )
            # configuration files: excluded file types
            if not os.path.exists(os_parityPath(os.path.join(config_dir,  _iam_script))):
                copyfile(
                    os_parityPath(os.path.join('bash', _iam_script)),
                    os_parityPath(os.path.join(config_dir, _iam_script))
                )
            # configuration files: excluded directories
            if not os.path.exists(os_parityPath(os.path.join(config_dir, _rgn_script))):
                copyfile(
                    os_parityPath(os.path.join('bash', _rgn_script)),
                    os_parityPath(os.path.join(config_dir, _rgn_script))
                )
        install.run(self)


def preclean(dst):
    if os.path.exists(dst):
        os.remove(dst)
    return True


def read(fname):
    basedir = os.path.dirname(sys.argv[0])
    return open(os.path.join(basedir, fname)).read()


def sourcefile_content():
    sourcefile = """
    for bcfile in ~/.bash_completion.d/* ; do
        [ -f "$bcfile" ] && . $bcfile
    done\n
    """
    return sourcefile


def user_home():
    """Returns os specific home dir for current user"""
    try:
        if platform.system() == 'Linux' or platform.system() == 'Darwin':
            # Linux or BSD Unix (Mac)
            return os.path.expanduser('~') or os.environ.get('HOME')

        elif platform.system() == 'Windows':
            username = os.getenv('username')
            return 'C:\\Users\\' + username

        elif platform.system() == 'Java':
            print('Unable to determine home dir, unsupported os type')
            sys.exit(1)
    except OSError as e:
        raise e


# branch install based on user priviledge level

if _root_user():

    setup(
        name=_project,
        version=rulemanager.__version__,
        description='Manage Amazon CloudWatch rules via cli',
        long_description=read('DESCRIPTION.rst'),
        url='https://bitbucket.org/the_organization/rulemanager',
        author=rulemanager.__author__,
        author_email=rulemanager.__email__,
        license='GPL-3.0',
        classifiers=[
            'Topic :: Software Development :: Build Tools',
            'Development Status :: 4 - Beta',
            'Programming Language :: Python :: 3.6',
            'Programming Language :: Python :: 3.7',
            'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
            'Operating System :: POSIX :: Linux',
        ],
        keywords='Amazon AWS ec2 cloudwatch rules events',
        packages=find_packages(exclude=['assets', 'docs', 'reports', 'scripts', 'tests']),
        include_package_data=True,
        install_requires=requires,
        python_requires='>=3.6, <4',
        cmdclass={
            'install': PostInstallRoot
        },
        data_files=[
            (
                os.path.join('/etc/bash_completion.d'),
                [os.path.join('bash', _comp_fname)]
            ),
            (
                os.path.join(user_home(), '.config', _project),
                [os.path.join('bash', _iam_script), os.path.join('bash', _rgn_script)]
            )
        ],
        entry_points={
            'console_scripts': [
                'rulemanager=rulemanager.cli:init'
            ]
        },
        zip_safe=False
    )

else:

    # non-priviledged user

    setup(
        name=_project,
        version=rulemanager.__version__,
        description='Manage Amazon CloudWatch rules via cli',
        long_description=read('DESCRIPTION.rst'),
        url='https://bitbucket.org/the_organization/rulemanager',
        author=rulemanager.__author__,
        author_email=rulemanager.__email__,
        license='GPL-3.0',
        classifiers=[
            'Topic :: Software Development :: Build Tools',
            'Development Status :: 4 - Beta',
            'Programming Language :: Python :: 3.6',
            'Programming Language :: Python :: 3.7',
            'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
            'Operating System :: POSIX :: Linux',
        ],
        keywords='Amazon AWS ec2 cloudwatch rules events',
        packages=find_packages(exclude=['assets', 'docs', 'reports', 'scripts', 'tests']),
        include_package_data=True,
        install_requires=requires,
        python_requires='>=3.6, <4',
        cmdclass={
            'install': PostInstall
        },
        data_files=[
            (
                os.path.join(user_home(), '.bash_completion.d'),
                [os.path.join('bash', _comp_fname)]
            ),
            (
                os.path.join(user_home(), '.config', _project),
                [os.path.join('bash', _iam_script)]
            ),
            (
                os.path.join(user_home(), '.config', _project),
                [os.path.join('bash',  _rgn_script)]
            )
        ],
        entry_points={
            'console_scripts': [
                'rulemanager=rulemanager.cli:init'
            ]
        },
        zip_safe=False
    )
